@@ -1,7 +1,13 @@
 from datasette import hookimpl
 from datasette.utils.asgi import Response, NotFound
-from datasette_tiles.utils import detect_mtiles_databases, tiles_stack_database_order
+from datasette_tiles.utils import (
+    detect_mtiles_databases,
+    tiles_stack_database_order,
+    latlon_to_tile_with_adjust,
+    tile_to_latlon,
+)
 import json
+import math
 
 # 256x256 PNG of colour #dddddd, compressed using https://squoosh.app
 PNG_404 = (
@@ -108,14 +114,26 @@ async def explorer(datasette, request):
     min_zoom = 0
     max_zoom = 19
     if metadata.get("minzoom"):
-        min_zoom = metadata["minzoom"]
+        min_zoom = int(metadata["minzoom"])
     if metadata.get("maxzoom"):
-        max_zoom = metadata["maxzoom"]
+        max_zoom = int(metadata["maxzoom"])
     attribution = metadata.get("attribution") or None
+
+    # Provided location data
+    lat = float(request.args.get("lat", default_latitude))
+    lon = float(request.args.get("lon", default_longitude))
+    zoom = int(request.args.get("z", default_zoom))
+    if zoom > max_zoom:
+        zoom = max_zoom
+    if zoom < min_zoom:
+        zoom = min_zoom
+    x_tile, y_tile = latlon_to_tile_with_adjust(lat, lon, zoom)
+
     return Response.html(
         await datasette.render_template(
             "tiles_explorer.html",
             {
+                "nojs": request.args.get("nojs") or request.args.get("lat"),
                 "metadata": metadata,
                 "db_name": db_name,
                 "db_path": datasette.urls.database(db_name),
@@ -125,6 +143,19 @@ async def explorer(datasette, request):
                 "min_zoom": min_zoom,
                 "max_zoom": max_zoom,
                 "attribution": json.dumps(attribution),
+                "current_latitude": lat,
+                "current_longitude": lon,
+                "can_zoom_in": zoom < max_zoom,
+                "can_zoom_out": zoom > min_zoom,
+                "current_zoom": zoom,
+                "current_x": x_tile,
+                "current_y": y_tile,
+                "compass": {
+                    "n": tile_to_latlon(x_tile, y_tile - 1, zoom),
+                    "s": tile_to_latlon(x_tile, y_tile + 1, zoom),
+                    "e": tile_to_latlon(x_tile + 1, y_tile, zoom),
+                    "w": tile_to_latlon(x_tile - 1, y_tile, zoom),
+                },
             },
         )
     )
